@@ -1,4 +1,4 @@
-RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv, t0.mle = NULL, t1.mle = NULL, mu0.mle = NULL,  mu1.mle = NULL, Sigma.mle = NULL, Sigma0.mle = NULL, Sigma1.mle = NULL, gam = NULL, ...) {
+RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv, t0.mle = NULL, t1.mle = NULL, mu0.mle = NULL,  mu1.mle = NULL, Sigma.mle = NULL, Sigma0.mle = NULL, Sigma1.mle = NULL, gam = NULL, kl.k = kl.k, ...) {
     list2env(list(...), environment())
     n <- length(ytrain)
     p <- ncol(xtrain)
@@ -6,7 +6,13 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
     p1 <- sum(ytrain == 1)/length(ytrain)
 
     if (base == "gamma") {
-
+        if (criterion == "nric") {
+            subspace.list <- sapply(1:B2, function(i) {
+                # the last row is training error for each i in 1:B2
+                Si <- S[, i][!is.na(S[, i])]  # current subspace
+                -2*(p0*KL.divergence(xtrain[ytrain == 0, Si, drop = F], xtrain[ytrain == 1, Si, drop = F], k = kl.k[1])[kl.k[1]] + p1*KL.divergence(xtrain[ytrain == 1, Si, drop = F], xtrain[ytrain == 0, Si, drop = F], k = kl.k[2])[kl.k[2]]) + length(Si)*2*log(log(n))/sqrt(n)
+            })
+        }
         if (criterion == "ric") {
             subspace.list <- sapply(1:B2, function(i) {
                 # the last row is training error for each i in 1:B2
@@ -50,6 +56,14 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
     }
 
     if (base == "logistic") {
+        if (criterion == "nric") {
+            subspace.list <- sapply(1:B2, function(i) {
+                # the last row is training error for each i in 1:B2
+                Si <- S[, i][!is.na(S[, i])]  # current subspace
+                -2*(p0*KL.divergence(xtrain[ytrain == 0, Si, drop = F], xtrain[ytrain == 1, Si, drop = F], k = kl.k[1])[kl.k[1]] + p1*KL.divergence(xtrain[ytrain == 1, Si, drop = F], xtrain[ytrain == 0, Si, drop = F], k = kl.k[2])[kl.k[2]]) + length(Si)*log(log(n))/sqrt(n)
+            })
+        }
+
         if (criterion == "ric") {
             subspace.list <- sapply(1:B2, function(i) {
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
@@ -58,7 +72,7 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 posterior0 <- 1/(1 + exp(score))
                 posterior1 <- 1 - posterior0
                 ric("other", xtrain, ytrain, Si, p0 = p0, p1 = p1, posterior0 = posterior0, posterior1 = posterior1, deg = function(i) {
-                  i
+                    i
                 })
             })
         }
@@ -76,7 +90,7 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 xtrain.r <- xtrain[, Si, drop = F]
                 mean(as.numeric(I(predict(glm(y ~ ., data = data.frame(x = xtrain.r, y = ytrain), family = "binomial"), data.frame(x = xtrain.r)) >
-                  0)) != ytrain, na.rm = TRUE)
+                                      0)) != ytrain, na.rm = TRUE)
             })
         }
 
@@ -87,7 +101,7 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 xtrain.r <- xtrain[, Si, drop = F]
                 xval.r <- xval[, Si, drop = F]
                 mean(as.numeric(I(predict(glm(y ~ ., data = data.frame(x = xtrain.r, y = ytrain), family = "binomial"), data.frame(x = xval.r)) >
-                  0)) != yval, na.rm = TRUE)
+                                      0)) != yval, na.rm = TRUE)
             })
         }
 
@@ -97,8 +111,8 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 # the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 mean(sapply(1:cv, function(j) {
-                  fit <- glm(y ~ ., data = data.frame(x = xtrain.r[-folds[[j]], Si, drop = F], y = ytrain[-folds[[j]]]), family = "binomial")
-                  mean(as.numeric(I(predict(fit, data.frame(x = xtrain.r[folds[[j]], Si, drop = F]))) > 0) != ytrain[folds[[j]]], na.rm = TRUE)
+                    fit <- glm(y ~ ., data = data.frame(x = xtrain[-folds[[j]], Si, drop = F], y = ytrain[-folds[[j]]]), family = "binomial")
+                    mean(as.numeric(I(predict(fit, data.frame(x = xtrain[folds[[j]], Si, drop = F]))) > 0) != ytrain[folds[[j]]], na.rm = TRUE)
                 }))
             })
         }
@@ -116,6 +130,14 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
     }
 
     if (base == "svm") {
+        if (criterion == "nric") {
+            subspace.list <- sapply(1:B2, function(i) {
+                # the last row is training error for each i in 1:B2
+                Si <- S[, i][!is.na(S[, i])]  # current subspace
+                -2*(p0*KL.divergence(xtrain[ytrain == 0, Si, drop = F], xtrain[ytrain == 1, Si, drop = F], k = kl.k)[kl.k] + p1*KL.divergence(xtrain[ytrain == 1, Si, drop = F], xtrain[ytrain == 0, Si, drop = F], k = kl.k)[kl.k]) +  + length(Si)*log(log(n))/sqrt(n)
+            })
+        }
+
         if (!is.character(kernel)) {
             kernel <- "linear"
         }
@@ -126,7 +148,7 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 xtrain.r <- xtrain[, Si, drop = F]
                 mean(as.numeric(predict(svm(x = xtrain.r, y = ytrain, kernel = kernel, type = "C-classification"), xtrain.r)) - 1 !=
-                  ytrain, na.rm = TRUE)
+                         ytrain, na.rm = TRUE)
             })
         }
 
@@ -137,7 +159,7 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 xtrain.r <- xtrain[, Si, drop = F]
                 xval.r <- xval[, Si, drop = F]
                 mean(as.numeric(predict(svm(x = xtrain.r, y = ytrain, kernel = kernel, type = "C-classification"), xval.r)) - 1 !=
-                  yval, na.rm = TRUE)
+                         yval, na.rm = TRUE)
             })
         }
 
@@ -147,8 +169,8 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 # the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 mean(sapply(1:cv, function(j) {
-                  fit <- svm(x = xtrain[-folds[[j]], Si, drop = F], y = ytrain[-folds[[j]]], kernel = kernel, type = "C-classification")
-                  mean(as.numeric(predict(fit, xtrain.r[folds[[j]], Si, drop = F])) - 1 != ytrain[folds[[j]]], na.rm = TRUE)
+                    fit <- svm(x = xtrain[-folds[[j]], Si, drop = F], y = ytrain[-folds[[j]]], kernel = kernel, type = "C-classification")
+                    mean(as.numeric(predict(fit, xtrain[folds[[j]], Si, drop = F])) - 1 != ytrain[folds[[j]]], na.rm = TRUE)
                 }))
             })
         }
@@ -200,8 +222,8 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 # the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 mean(sapply(1:cv, function(j) {
-                  fit <- randomForest(x = xtrain[-folds[[j]], Si, drop = F], y = factor(ytrain[-folds[[j]]]))
-                  mean((as.numeric(predict(fit, xtrain[folds[[j]], Si, drop = F])) - 1) != ytrain[folds[[j]]], na.rm = TRUE)
+                    fit <- randomForest(x = xtrain[-folds[[j]], Si, drop = F], y = factor(ytrain[-folds[[j]]]))
+                    mean((as.numeric(predict(fit, xtrain[folds[[j]], Si, drop = F])) - 1) != ytrain[folds[[j]]], na.rm = TRUE)
                 }))
             })
         }
@@ -233,7 +255,7 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 Si <- matrix(S[, i][!is.na(S[, i])], nrow = d)  # current subspace
                 xtrain.r <- xtrain[, Si, drop = F]
                 knn.test <- sapply(k, function(j) {
-                  mean(knn.cv(xtrain.r, ytrain, j, use.all = FALSE) != ytrain, na.rm = TRUE)
+                    mean(knn.cv(xtrain.r, ytrain, j, use.all = FALSE) != ytrain, na.rm = TRUE)
                 })
                 min(knn.test)
             })
@@ -259,8 +281,8 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 xval.r <- xval[, Si, drop = F]
 
                 knn.test <- sapply(k, function(j) {
-                  fit <- knn3(x = xtrain.r, y = factor(ytrain), k = j, use.all = FALSE)
-                  mean(predict(fit, xval.r, type = "class") != yval)
+                    fit <- knn3(x = xtrain.r, y = factor(ytrain), k = j, use.all = FALSE)
+                    mean(predict(fit, xval.r, type = "class") != yval)
                 })
                 min(knn.test)
             })
@@ -294,6 +316,7 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
     }
 
     if (base == "tree") {
+
         if (criterion == "training") {
             subspace.list <- sapply(1:B2, function(i) {
                 # the last row is training error for each i in 1:B2
@@ -321,9 +344,9 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 # the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 mean(sapply(1:cv, function(j) {
-                  fit <- rpart(y ~ ., data = data.frame(x = xtrain[-folds[[j]], Si, drop = F], y = ytrain[-folds[[j]]]), method = "class")
-                  mean((as.numeric(predict(fit, data = data.frame(x = xtrain[folds[[j]], Si, drop = F]), type = "class")) - 1) != ytrain[folds[[j]]],
-                    na.rm = TRUE)
+                    fit <- rpart(y ~ ., data = data.frame(x = xtrain[-folds[[j]], Si, drop = F], y = ytrain[-folds[[j]]]), method = "class")
+                    mean((as.numeric(predict(fit, data = data.frame(x = xtrain[folds[[j]], Si, drop = F]), type = "class")) - 1) != ytrain[folds[[j]]],
+                         na.rm = TRUE)
                 }))
             })
         }
@@ -349,6 +372,14 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
     }
 
     if (base == "lda") {
+        if (criterion == "nric") {
+            subspace.list <- sapply(1:B2, function(i) {
+                # the last row is training error for each i in 1:B2
+                Si <- S[, i][!is.na(S[, i])]  # current subspace
+                -2*(p0*KL.divergence(xtrain[ytrain == 0, Si, drop = F], xtrain[ytrain == 1, Si, drop = F], k = kl.k[1])[kl.k[1]] + p1*KL.divergence(xtrain[ytrain == 1, Si, drop = F], xtrain[ytrain == 0, Si, drop = F], k = kl.k[2])[kl.k[2]]) + length(Si)*log(log(n))/sqrt(n)
+            })
+        }
+
         if (criterion == "ric") {
             subspace.list <- sapply(1:B2, function(i) {
                 # the last row is training error for each i in 1:B2
@@ -390,8 +421,8 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 # the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 mean(sapply(1:cv, function(j) {
-                  mean(predict(lda(x = xtrain[-folds[[j]], Si, drop = F], grouping = ytrain[-folds[[j]]]), xtrain[folds[[j]], Si, drop = F])$class !=
-                    ytrain[folds[[j]]], na.rm = TRUE)
+                    mean(predict(lda(x = xtrain[-folds[[j]], Si, drop = F], grouping = ytrain[-folds[[j]]]), xtrain[folds[[j]], Si, drop = F])$class !=
+                             ytrain[folds[[j]]], na.rm = TRUE)
                 }))
             })
         }
@@ -410,12 +441,20 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
     }
 
     if (base == "qda") {
+        if (criterion == "nric") {
+            subspace.list <- sapply(1:B2, function(i) {
+                # the last row is training error for each i in 1:B2
+                Si <- S[, i][!is.na(S[, i])]  # current subspace
+                -2*(p0*KL.divergence(xtrain[ytrain == 0, Si, drop = F], xtrain[ytrain == 1, Si, drop = F], k = kl.k[1])[kl.k[1]] + p1*KL.divergence(xtrain[ytrain == 1, Si, drop = F], xtrain[ytrain == 0, Si, drop = F], k = kl.k[2])[kl.k[2]]) + length(Si)*(length(Si) + 3)/2*log(log(n))/sqrt(n)
+            })
+        }
+
         if (criterion == "ric") {
             subspace.list <- sapply(1:B2, function(i) {
                 # print(i) the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 ric("qda", xtrain, ytrain, Si, mu0.mle = mu0.mle, mu1.mle = mu1.mle, Sigma0.mle = Sigma0.mle, Sigma1.mle = Sigma1.mle,
-                  p0 = p0, p1 = p1)
+                    p0 = p0, p1 = p1)
             })
         }
 
@@ -450,8 +489,8 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
             subspace.list <- sapply(1:B2, function(i) {
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 mean(sapply(1:cv, function(j) {
-                  mean(predict(qda(x = xtrain[-folds[[j]], Si, drop = F], grouping = ytrain[-folds[[j]]]), xtrain[folds[[j]], Si, drop = F])$class !=
-                    ytrain[folds[[j]]], na.rm = TRUE)
+                    mean(predict(qda(x = xtrain[-folds[[j]], Si, drop = F], grouping = ytrain[-folds[[j]]]), xtrain[folds[[j]], Si, drop = F])$class !=
+                             ytrain[folds[[j]]], na.rm = TRUE)
                 }))
             })
         }
